
/**
 * Module dependencies.
 */

var debug = require('debug')('s3-aggregate');
var Emitter = require('events').EventEmitter;
var assert = require('assert');
var Batch = require('batch');
var zlib = require('zlib');
var uid = require('uid2');
var fs = require('fs');

/**
 * Expose `Aggregator`.
 */

module.exports = Aggregator;

/**
 * Initialize a new aggregator.
 *
 * @param {Object} opts
 * @api public
 */

function Aggregator(opts) {
  if (!(this instanceof Aggregator)) return new Aggregator(opts);
  this.required = opts.required;
  this.client = opts.client;
  this.files = opts.files;
  this.path = opts.path;
  this.part = 0;
  this.parts = [];
  this.id = uid(30);
  assert(this.path, '.path required');
  assert(this.client, '.client required');
  assert(this.files, '.files required');
  this.start();
}

/**
 * Inherit from `Emitter.prototype`.
 */

Aggregator.prototype.__proto__ = Emitter.prototype;

/**
 * Start fetching files.
 *
 * @api private
 */

Aggregator.prototype.start = function(){
  var batch = new Batch;
  var self = this;

  batch.concurrency(5);

  batch.on('progress', function(e){
    self.emit('progress', e);
  });

  this.files.forEach(function(file){
    batch.push(function(done){
      self.get(file, done);
    });
  });

  debug('streaming %j', this.files);
  batch.end(function(err){
    if (err) return self.emit('error', err);
    self.concat();
  });
};

Aggregator.prototype.concat = function(){
  var batch = new Batch;
  var path = this.path;
  var self = this;

  debug('concatenating %s parts to %s', this.parts.length, path);
  var gzip = zlib.createGzip();
  var out = fs.createWriteStream(path);
  gzip.pipe(out);

  batch.concurrency(1);

  this.parts.forEach(function(part){
    batch.push(function(done){
      debug('concat %s', part);
      var file = fs.createReadStream(part);
      file.pipe(gzip, { end: false });
      file.on('error', done);
      file.on('end', function(){
        fs.unlink(part);
        gzip.write('\n', done);
      });
    });
  });

  batch.end(function(err){
    if (err) return self.emit('error', err);
    gzip.end();
    gzip.on('close', function(){
      // give shitty node time to flush to disk
      setTimeout(function(){
        self.emit('end');
      }, 10000);
    });
  });
};

/**
 * Get `file`.
 *
 * @param {String} file
 * @param {Function} fn
 * @api private
 */

Aggregator.prototype.get = function(file, fn){
  debug('get %s', file);
  var req = this.client.get(file);
  var out = fs.createWriteStream('/tmp/' + this.id + '-' + this.part++ + '.part');
  this.parts.push(out.path);
  var self = this;

  req.on('response', function(res){
    debug('%s response %s', res.statusCode, file);
    var s = res.statusCode / 100 | 0;

    if (4 == s) {
      if (self.required) {
        var err = new Error(res.statusCode + ' response');
        err.status = res.statusCode;
        err.file = file;
        return fn(err);
      } else {
        self.emit('failed', { file: file, status: res.statusCode });
        return fn();
      }
    }

    var gunzip = zlib.createGunzip();
    res.pipe(gunzip).pipe(out, { end: false });
    gunzip.on('error', fn);
    out.on('error', fn);
    res.on('error', fn);
    gunzip.on('finish', fn);
  });

  req.end();
};